/******************************************************************************

  Copyright(c) 2003 - 2007 Intel Corporation. All rights reserved.

  This program is free software; you can redistribute it and/or modify it
  under the terms of version 2 of the GNU General Public License as
  published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110, USA

  The full GNU General Public License is included in this distribution in the
  file called LICENSE.

  Contact Information:
  James P. Ketrenos <ipw2100-admin@linux.intel.com>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

******************************************************************************/

#ifndef __iwl_3945_rs_h__
#define __iwl_3945_rs_h__

#include "iwl-hw.h"

struct ipw_rate_info {
	u8 plcp;
	u8 ieee;
	u8 prev_ieee;    /* previous rate in IEEE speeds */
	u8 next_ieee;    /* next rate in IEEE speeds */
	u8 prev_rs;      /* previous rate used in rs algo */
	u8 next_rs;      /* next rate used in rs algo */
	u8 prev_rs_tgg;  /* previous rate used in TGG rs algo */
	u8 next_rs_tgg;  /* next rate used in TGG rs algo */
};

enum {
	IPW_RATE_6M_INDEX = 0,
	IPW_RATE_9M_INDEX,
	IPW_RATE_12M_INDEX,
	IPW_RATE_18M_INDEX,
	IPW_RATE_24M_INDEX,
	IPW_RATE_36M_INDEX,
	IPW_RATE_48M_INDEX,
	IPW_RATE_54M_INDEX,
	IPW_RATE_1M_INDEX,
	IPW_RATE_2M_INDEX,
	IPW_RATE_5M_INDEX,
	IPW_RATE_11M_INDEX,
	IPW_RATE_COUNT,
	IPW_RATE_INVM_INDEX,
	IPW_RATE_INVALID = IPW_RATE_INVM_INDEX
};

enum {
	IPW_FIRST_OFDM_RATE = IPW_RATE_6M_INDEX,
	IPW_LAST_OFDM_RATE = IPW_RATE_54M_INDEX,
	IPW_FIRST_CCK_RATE = IPW_RATE_1M_INDEX,
	IPW_LAST_CCK_RATE = IPW_RATE_11M_INDEX,
};

/* #define vs. enum to keep from defaulting to 'large integer' */
#define	IPW_RATE_6M_MASK   (1<<IPW_RATE_6M_INDEX)
#define	IPW_RATE_9M_MASK   (1<<IPW_RATE_9M_INDEX)
#define	IPW_RATE_12M_MASK  (1<<IPW_RATE_12M_INDEX)
#define	IPW_RATE_18M_MASK  (1<<IPW_RATE_18M_INDEX)
#define	IPW_RATE_24M_MASK  (1<<IPW_RATE_24M_INDEX)
#define	IPW_RATE_36M_MASK  (1<<IPW_RATE_36M_INDEX)
#define	IPW_RATE_48M_MASK  (1<<IPW_RATE_48M_INDEX)
#define	IPW_RATE_54M_MASK  (1<<IPW_RATE_54M_INDEX)
#define	IPW_RATE_1M_MASK   (1<<IPW_RATE_1M_INDEX)
#define	IPW_RATE_2M_MASK   (1<<IPW_RATE_2M_INDEX)
#define	IPW_RATE_5M_MASK   (1<<IPW_RATE_5M_INDEX)
#define	IPW_RATE_11M_MASK  (1<<IPW_RATE_11M_INDEX)

enum {
	IPW_RATE_6M_PLCP  = 13,
	IPW_RATE_9M_PLCP  = 15,
	IPW_RATE_12M_PLCP = 5,
	IPW_RATE_18M_PLCP = 7,
	IPW_RATE_24M_PLCP = 9,
	IPW_RATE_36M_PLCP = 11,
	IPW_RATE_48M_PLCP = 1,
	IPW_RATE_54M_PLCP = 3,
	IPW_RATE_1M_PLCP  = 10,
	IPW_RATE_2M_PLCP  = 20,
	IPW_RATE_5M_PLCP  = 55,
	IPW_RATE_11M_PLCP = 110,
};

enum {
	IPW_RATE_6M_IEEE  = 12,
	IPW_RATE_9M_IEEE  = 18,
	IPW_RATE_12M_IEEE = 24,
	IPW_RATE_18M_IEEE = 36,
	IPW_RATE_24M_IEEE = 48,
	IPW_RATE_36M_IEEE = 72,
	IPW_RATE_48M_IEEE = 96,
	IPW_RATE_54M_IEEE = 108,
	IPW_RATE_1M_IEEE  = 2,
	IPW_RATE_2M_IEEE  = 4,
	IPW_RATE_5M_IEEE  = 11,
	IPW_RATE_11M_IEEE = 22,
};

#define IPW_CCK_BASIC_RATES_MASK    \
       (IPW_RATE_1M_MASK          | \
        IPW_RATE_2M_MASK)

#define IPW_CCK_RATES_MASK          \
       (IPW_BASIC_RATES_MASK      | \
        IPW_RATE_5M_MASK          | \
        IPW_RATE_11M_MASK)

#define IPW_OFDM_BASIC_RATES_MASK   \
        (IPW_RATE_6M_MASK         | \
        IPW_RATE_12M_MASK         | \
        IPW_RATE_24M_MASK)

#define IPW_OFDM_RATES_MASK         \
       (IPW_OFDM_BASIC_RATES_MASK | \
        IPW_RATE_9M_MASK          | \
        IPW_RATE_18M_MASK         | \
        IPW_RATE_36M_MASK         | \
        IPW_RATE_48M_MASK         | \
        IPW_RATE_54M_MASK)

#define IPW_BASIC_RATES_MASK         \
        (IPW_OFDM_BASIC_RATES_MASK | \
         IPW_CCK_BASIC_RATES_MASK)

#define IPW_RATES_MASK ((1<<IPW_RATE_COUNT)-1)

#define IPW_INVALID_VALUE    -1

#define IPW_MIN_RSSI_VAL                 -100
#define IPW_MAX_RSSI_VAL                    0

extern const struct ipw_rate_info ipw_rates[IPW_RATE_COUNT];

static inline int ipw_rate_index_from_plcp(int plcp)
{
	int i = 0;
	for (i = 0; i < ARRAY_SIZE(ipw_rates); i++) 
		if (ipw_rates[i].plcp == plcp)
			return i;
	return -1;
}

struct ipw_priv;
extern int ipw_rate_scale_rxon_handle(struct ipw_priv *priv, s32 sta_id);
extern void ipw_set_rate_scale_handlers(struct rate_control_ops *rs_ops);
extern int ipw_fill_rs_info(struct ipw_priv *priv, char *buf, u8 sta_id);
extern u8 ipw_get_prev_ieee_rate(u8 rate);
#endif
